function [error, psnr_all, psnr_mask, ssim_all, ssim_mask] = recover_mrf_fast(im, im_ori, masks, save_dir, batchID, method)
if ~exist(save_dir, 'dir')
    mkdir(save_dir);
end
im = gather(im);
im_ori = gather(im_ori);
masks = gather(masks);

num_sweep = 500;

    
if method == 1
    beta = 0.0005;
else
    beta = sqrt(0.0005);
end

img_result=zeros(size(im));
parfor i = 1: size(im, 4)
    fprintf('MRF recovery, image #%d\n', i);
    img = (im(:,:,:,i)+1)/2 * 255;
    img = padarray(img, [1, 1]);
    mask = masks(:,:,1, i);
    mask = padarray(mask, [1, 1]);
    
    img_ori = im_ori(:,:,:,i);
    img_ori = padarray(img_ori, [1, 1]);
    idx = find(mask == 1);
    [idx_r, idx_c] = ind2sub(size(mask), idx);
    for t = 1:num_sweep
        for ch = 1:size(img, 3)
            R_I = img(:,:,ch);
            for j = 1:numel(idx_r)
               r = idx_r(j);
               c = idx_c(j);
               R_I(r,c) = Gibbs(r, c, R_I, beta, method);
            end
            img(:,:,ch) = R_I;
        end
    end
    
    img_result(:,:,:,i)=img(2:end-1,2:end-1,:)/255 * 2 -1;    
end

error = 0;
psnr_all =0;
psnr_mask = 0;

ssim_all = 0;
ssim_mask = 0;

for i = 1: size(im, 4)
    
    img=img_result(:,:,:,i);
    mask2 = masks(:,:,:,i);
    img_ori=im_ori(:,:,:,i);
    
   % img = img/255 * 2 -1;

    img = (img - min(img(:))) / (max(img(:)) - min(img(:)));    
    imwrite(img, [save_dir, 'batch_', num2str(batchID), num2str(i, '_recover_%04d.png')]);
    re_img_255=uint8(img*255);
    
    img1 = (img_ori - min(img_ori(:))) / (max(img_ori(:)) - min(img_ori(:)));
    ori_img_255=uint8(img1*255);
    
    error_temp=mean(reshape(abs(re_img_255(mask2==1) - ori_img_255(mask2==1)), [], 1));    
    error = error + error_temp;
    
    psnr_all = psnr_all + psnr(re_img_255, ori_img_255);
    psnr_mask = psnr_mask + psnr(re_img_255(mask2==1), ori_img_255(mask2==1));
    
    ssim_all = ssim_all + ssim(re_img_255, ori_img_255);
    ssim_mask = ssim_mask + ssim(re_img_255(mask2==1), ori_img_255(mask2==1));   
    
    disp(['image', num2str(i)]);
end

error = (error/255.0) / size(im, 4);
psnr_all = psnr_all / size(im, 4);
psnr_mask = psnr_mask / size(im, 4);
ssim_all = ssim_all / size(im, 4);
ssim_mask = ssim_mask / size(im, 4);
    


function [pix]=Gibbs(x,y,im,beta,method)
% pix= 0-255
pix = -1;

i=0:1:255;

switch method
    case 1
        X=exp (-beta* ( (im(x+1,y)-i).^2 + (im(x,y+1)-i).^2 + (im(x-1,y)-i).^2 + (im(x,y-1)-i).^2 + (im(x-1,y-1)-i).^2 + (im(x-1,y+1)-i).^2 + (im(x+1,y-1)-i).^2 + (im(x+1,y+1)-i).^2 ));
    case 2
        X=exp (-beta* ( abs(im(x+1,y)-i) + abs(im(x,y+1)-i) + abs(im(x-1,y)-i) + abs(im(x,y-1)-i) + abs(im(x-1,y-1)-i) + abs(im(x-1,y+1)-i) + abs(im(x+1,y-1)-i) + abs(im(x+1,y+1)-i) ));
    otherwise
        error('No such a method');
end
z=sum(X);
X=X./z;
CDF_X=cumsum(X);

ra=rand();
for i=1:256
    if ra<=CDF_X(i)
        pix=i-1;
        break;
    end
end