function [net1, net2, net3, opts] = train_model_multigrid(opts, net1, net2, net3, imdb, getBatch)

rate_list = logspace(-2, -4, 80)*100;
learningRate_array1 = repmat(rate_list , max(1,floor(opts.numEpochs / length(rate_list))),1); %logspace(-2, -4, 60) ones(1,60, 'single')
learningRate_array1 = reshape(learningRate_array1, 1, []);
learningRate_array2 = learningRate_array1;

rate_list = logspace(-2, -3, 40)*100;
learningRate_array3 = repmat(rate_list , max(1,floor(opts.numEpochs / length(rate_list))),1); %logspace(-2, -4, 60) ones(1,60, 'single')
learningRate_array3 = reshape(learningRate_array3, 1, []);

opts.train = [] ;
opts.val = [] ;
opts.conserveMemory = true ;
opts.backPropDepth = +inf ;
opts.sync = false ;
opts.prefetch = false;
opts.numFetchThreads = 8;
opts.cudnn = true ;
opts.weightDecay = 0.0001 ; %0.0001
opts.momentum = 0.9;
opts.learningRate1 = reshape(learningRate_array1, 1, []);
opts.learningRate2 = reshape(learningRate_array2, 1, []);
opts.learningRate3 = reshape(learningRate_array3, 1, []);

if isempty(opts.train), opts.train = find(imdb.images.set==1) ; end
if isempty(opts.val), opts.val = find(imdb.images.set==2) ; end
if isnan(opts.train), opts.train = [] ; end

opts.batch_size = min(opts.batch_size, numel(opts.train));

% net1 = initialize_momentum(net1);
% net2 = initialize_momentum(net2);
% net3 = initialize_momentum(net3);


interval = 10;
SSD1 = zeros(opts.numEpochs, 1);
SSD2 = zeros(opts.numEpochs, 1);
SSD3 = zeros(opts.numEpochs, 1);

mean_img1 = net1.normalization.averageImage;
mean_img2 = net2.normalization.averageImage;
mean_img3 = net3.normalization.averageImage;

batchStart = 1;
batchEnd = min(1+opts.batch_size-1, numel(opts.train)) ;
batch = opts.train(batchStart : batchEnd) ;
% The 1 in mask means the block area
[im, ori_im, ~, mask] = getBatch(imdb, batch);
draw_figures(opts, ori_im, 0, mean_img3, [], 'original', mask);
draw_figures(opts, im, 0, mean_img3, [], 'masked', mask);


% -------------------------------------------------------------------------
%                           Train and validate
% -------------------------------------------------------------------------

model_file = [opts.working_folder, 'iter_',...
    num2str(opts.numEpochs) ,'_model.mat'];

if exist(model_file, 'file') && opts.force_learn == false;
    load(model_file);
else
%     h = figure;
    for epoch=1:opts.numEpochs
        fprintf('Iteration %d / %d\n', epoch, opts.numEpochs);
        
        learningRate = [opts.learningRate1(min(epoch, numel(opts.learningRate1))), ...
            opts.learningRate2(min(epoch, numel(opts.learningRate2))), ...
            opts.learningRate3(min(epoch, numel(opts.learningRate3)))];
        train = opts.train;
        
        [net1, net2, net3, syn_mats1, syn_mats2, syn_mats3, loss, mask1, mask2, mask3, ori_img_mats] = ...
            process_epoch_multigrid3(opts, getBatch, epoch, train, learningRate, imdb, net1, net2, net3);

        % Calculating the psnr and ssim metrics to compare with baseline
        get_recover_metrics(opts, syn_mats3, ori_img_mats, mask3);
        
        SSD1(epoch) = loss(1);
        SSD2(epoch) = loss(2);
        SSD3(epoch) = loss(3);
        
        disp(['Loss: ', num2str(SSD1(epoch))]);
        if mod(epoch - 1, interval) == 0 || epoch == opts.numEpochs
            draw_figures(opts, syn_mats1{1}, epoch, mean_img1, SSD1, 'net1', mask1{1});
            draw_figures(opts, syn_mats2{1}, epoch, mean_img2, SSD2, 'net2', mask2{1});
            draw_figures(opts, syn_mats3{1}, epoch, mean_img3, SSD3, 'net3', mask3{1});
            
            if mod(epoch - 1, interval*5) == 0 || epoch == opts.numEpochs
                
                model_file = [opts.working_folder, 'iter_',...
                    num2str(epoch) ,'_model.mat'];
                save(model_file, 'net1', 'net2', 'net3', 'opts');
                
                % saveas(h, [opts.working_folder, 'iter_',...
                %    num2str(epoch) ,'_error.fig']);
%                 saveas(h, [opts.working_folder, 'iter_',...
%                     num2str(epoch) ,'_error.png'])
            end
        end
    end
end
end

function net = initialize_momentum(net)
for i=1:numel(net.layers)
    if isfield(net.layers{i}, 'weights')
        J = numel(net.layers{i}.weights) ;
        for j=1:J
            net.layers{i}.momentum{j} = zeros(size(net.layers{i}.weights{j}), 'single') ;
        end
        if ~isfield(net.layers{i}, 'learningRate')
            net.layers{i}.learningRate = ones(1, J, 'single') ;
        end
        if ~isfield(net.layers{i}, 'weightDecay')
            net.layers{i}.weightDecay = ones(1, J, 'single') ;
        end
    end
end
end