function [ psnr_mask, psnr_all, ssim_mask, ssim_all ] = get_recover_metrics( opts, rec_img_mats, ori_img_mats, masks_mats )

recons_err = 0.0;
psnr_mask = 0.0;
psnr_all = 0.0;
ssim_mask = 0.0;
ssim_all = 0.0;

if size(rec_img_mats,2) ~= size(ori_img_mats, 2)
    error 'Not the same size of recover image and original image!';
end

batch_num = size(rec_img_mats, 2);

for iBatch = 1:batch_num
    rec_img_mat = uint8(rec_img_mats{iBatch});
    ori_img_mat = uint8(ori_img_mats{iBatch});
    masks = masks_mats{iBatch};
    for iImg = 1:size(rec_img_mat, 4)
        rec_img = rec_img_mat(:,:,:,iImg);
        ori_img = ori_img_mat(:,:,:,iImg);
        mask = masks(:,:,:,iImg);
        recons_err = recons_err + mean(reshape(abs(rec_img(mask == 1) - ori_img(mask == 1)), [], 1));
        psnr_mask = psnr_mask + psnr(rec_img(mask == 1), ori_img(mask == 1));
        psnr_all = psnr_all + psnr(rec_img, ori_img);
        ssim_mask = ssim_mask + ssim(rec_img(mask == 1), ori_img(mask == 1));
        ssim_all = ssim_all + ssim(rec_img, ori_img);
    end
end

recons_err = (recons_err/255.0)/size(rec_img_mats{1}, 4)/batch_num;
psnr_mask = psnr_mask/size(rec_img_mats{1}, 4)/batch_num;
psnr_all = psnr_all/size(rec_img_mats{1}, 4)/batch_num;
ssim_mask = ssim_mask/size(rec_img_mats{1}, 4)/batch_num;
ssim_all = ssim_all/size(rec_img_mats{1}, 4)/batch_num;

fid = fopen([opts.Synfolder, 'model', num2str(opts.model), '_', num2str(opts.T,'%.1f'), '_', num2str(opts.Delta,'%.1f'), '_', ...
    num2str(opts.Gamma,'%.1f'), '_', opts.doodle_name, '_thre', num2str(opts.mask_thre), ...
    '_eachstep', num2str(opts.scale_eachstep), '_onlymask', num2str(opts.scale_onlymask), '_recover_metrics.txt'], 'w');

fprintf(fid, 'error %.4f\n', recons_err);
fprintf(fid, 'psnr_mask %.4f\n', psnr_mask);
fprintf(fid, 'psnr_all %.4f\n', psnr_all);
fprintf(fid, 'ssim_mask %.4f\n', ssim_mask);
fprintf(fid, 'ssim_all %.4f\n\n', ssim_all);

end

