function [net, syn_mat, res, res_syn, loss_temp] = update_network(net, syn_mat, im, res, res_syn, opts, lr, mask)
    numImages = size(im, 4);
    dydz = gpuArray(ones(net.dydz_sz, 'single'));
    dydz = repmat(dydz, 1, 1, 1, numImages);
    for tt = 1:opts.T
        res = vl_simplenn(net, syn_mat, dydz, res, ...
            'accumulate', false, ...
            'disableDropout', false, ...
            'conserveMemory', opts.conserveMemory, ...
            'backPropDepth', opts.backPropDepth, ...
            'sync', opts.sync, ...
            'cudnn', opts.cudnn);
        
        % Only update the field masked by mask
        back_up = syn_mat;
        % y^(t+1)
        syn_mat = syn_mat +  opts.Delta^2/2 * (res(1).dzdx - syn_mat / opts.refsig^2)+ ...
            opts.Delta * gpuArray(randn(size(syn_mat), 'single'));
        syn_mat(mask==0) = back_up(mask==0);
    end
    
%     loss = loss + gather( mean(reshape(abs(syn_mat - im), [], 1))) / num_cell / 2;    
    
    res = vl_simplenn(net, im, dydz, res, ...
        'accumulate', false, ...
        'disableDropout', false, ...
        'conserveMemory', opts.conserveMemory, ...
        'backPropDepth', opts.backPropDepth, ...
        'sync', opts.sync, ...
        'cudnn', opts.cudnn);
    
    res_syn = vl_simplenn(net, syn_mat, dydz, res_syn, ...
        'accumulate', false, ...
        'disableDropout', false, ...
        'conserveMemory', opts.conserveMemory, ...
        'backPropDepth', opts.backPropDepth, ...
        'sync', opts.sync, ...
        'cudnn', opts.cudnn);   
        
    [net, loss_temp] = accumulate_gradients(opts, lr * opts.Gamma, opts.batchSize, net, res, res_syn);
end

function [net, loss] = accumulate_gradients(opts, lr, batchSize, net, res, res_syn)
% -------------------------------------------------------------------------
layer_sets = net.layer_sets;
loss = [];
n = 0;
for l = layer_sets
    for j=1:numel(res(l).dzdw)
        thisDecay = opts.weightDecay * net.layers{l}.weightDecay(j) ;
        thisLR = lr * net.layers{l}.learningRate(j) ;
        
        if isfield(net.layers{l}, 'weights')
            gradient_dzdw = ((1 / batchSize) * res(l).dzdw{j} -  ...
                (1 / batchSize) * res_syn(l).dzdw{j}) / net.numFilters(l);
            if max(abs(gradient_dzdw(:))) > opts.cap %10
                gradient_dzdw = gradient_dzdw / max(abs(gradient_dzdw(:))) * opts.cap;
            end
            
          
            n = n+1;
            
            net.layers{l}.momentum{j} = ...
                + opts.momentum * net.layers{l}.momentum{j} ...
                - thisDecay * net.layers{l}.weights{j} ...
                + gradient_dzdw;
            
            loss = [loss, gather(mean(abs(gradient_dzdw(:))))];

            net.layers{l}.weights{j} = net.layers{l}.weights{j} + thisLR *net.layers{l}.momentum{j};
            
            if j == 1
                res_l = min(l+2, length(res));
%                 fprintf('Net: layer %s:max response is %f, min response is %f.\n', net.layers{l}.name, max(res(res_l).x(:)), min(res(res_l).x(:)));
%                 fprintf('max gradient is %f, min gradient is %f, learning rate is %f\n', max(gradient_dzdw(:)), min(gradient_dzdw(:)), thisLR);
            end
        end
    end
end
loss = mean(loss);
end